functions {
  real[] onecomp_mm_infusion(real t,
                             real[] y,
                             real[] theta,
                             real[] x_r,
                             int[] x_i) {

    real dydt[1];

    if (t < x_r[2])
      dydt[1] <- - theta[2] * y[1] / ( theta[1] * (theta[3] + y[1]) ) + x_r[1] / (theta[1] * x_r[2]);
    else
      dydt[1] <- - theta[2] * y[1] / ( theta[1] * (theta[3] + y[1]) );

    return dydt;

  }

}

data {
  int N_t;
  int N_patients;
  real C_hat[N_patients, N_t];
}

transformed data {
  real t[N_t];
  real t0;

  real C0[1];

  real x_r[2];
  int x_i[0];

  t[1] <- 0.1;
  t[2] <- 0.180164823065;
  t[3] <- 0.324593634702;
  t[4] <- 0.584803547643;
  t[5] <- 1.05361027689;
  t[6] <- 1.89823509116;
  t[7] <- 3.41995189335;
  t[8] <- 6.16155027758;
  t[9] <- 11.1009461557;
  t[10] <- 20.0;

  C0[1] <- 0;

  x_r[1] <- 15; // Total dosage in mg
  x_r[2] <- 5; // Infusion time in days

}

parameters {
  real eta_V[N_patients];
  real mu_V;
  real<lower=0> sigma_V;
  
  real eta_Cl_max[N_patients];
  real mu_Cl_max;
  real<lower=0> sigma_Cl_max;
  
  real<lower=0> C50;
}

transformed parameters {
  real<lower=0> V[N_patients];
  real<lower=0> Cl_max[N_patients];
  
  real C[N_patients, N_t];

  for (p in 1:N_patients) {
    real C_int[N_t, 1];
    real theta[3];
    
    V[p] <- exp(mu_V + eta_V[p] * sigma_V);
    Cl_max[p] <- exp(mu_Cl_max + eta_Cl_max[p] * sigma_Cl_max);
    
    theta[1] <- V[p];
    theta[2] <- Cl_max[p];
    theta[3] <- C50;

    C_int <- integrate_ode(onecomp_mm_infusion, C0, t0, t, theta, x_r, x_i);

    for (n in 1:N_t) C[p, n] <- C_int[n, 1];
  }
}

model {
  
  eta_V ~ normal(0, 1);
  mu_V ~ normal(log(5), 1);
  sigma_V ~ cauchy(0, 1);
  
  eta_Cl_max ~ normal(0, 1);
  mu_Cl_max ~ cauchy(0, 1);
  sigma_Cl_max ~ cauchy(0, 1);
  
  C50 ~ cauchy(0, 1);

  for (p in 1:N_patients)
    for (n in 1:N_t)
      C_hat[p, n] ~ lognormal(log(C[p, n]), 0.15);
  //C_hat[p] ~ lognormal(log(to_vector(C[p])), 0.15);
}
    
generated quantities {
  real C_ppc[N_patients, N_t];
  for (p in 1:N_patients)
    for (n in 1:N_t)
      C_ppc[p, n] <- lognormal_rng(log(C[p, n]), 0.15);

}
